@@ -1,9 +1,12 @@
-# Copyright (C) 2013-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2013-2017 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
-from sqlalchemy.schema import ColumnCollectionConstraint
-from sqlalchemy.sql import expression
+from ...sql.schema import ColumnCollectionConstraint
+from ...sql import expression
+from ... import util
+
 
 class ExcludeConstraint(ColumnCollectionConstraint):
     """A table-level EXCLUDE constraint.
@@ -11,7 +14,8 @@ class ExcludeConstraint(ColumnCollectionConstraint):
     Defines an EXCLUDE constraint as described in the `postgres
     documentation`__.
 
-    __ http://www.postgresql.org/docs/9.0/static/sql-createtable.html#SQL-CREATETABLE-EXCLUDE
+    __ http://www.postgresql.org/docs/9.0/\
+static/sql-createtable.html#SQL-CREATETABLE-EXCLUDE
     """
 
     __visit_name__ = 'exclude_constraint'
@@ -19,7 +23,7 @@ class ExcludeConstraint(ColumnCollectionConstraint):
     where = None
 
     def __init__(self, *elements, **kw):
-        """
+        r"""
         :param \*elements:
           A sequence of two tuples of the form ``(column, operator)`` where
           column must be a column name or Column object and operator must
@@ -45,29 +49,50 @@ class ExcludeConstraint(ColumnCollectionConstraint):
           for this constraint.
 
         """
+        columns = []
+        render_exprs = []
+        self.operators = {}
+
+        expressions, operators = zip(*elements)
+
+        for (expr, column, strname, add_element), operator in zip(
+                self._extract_col_expression_collection(expressions),
+                operators
+        ):
+            if add_element is not None:
+                columns.append(add_element)
+
+            name = column.name if column is not None else strname
+
+            if name is not None:
+                # backwards compat
+                self.operators[name] = operator
+
+            expr = expression._literal_as_text(expr)
+
+            render_exprs.append(
+                (expr, name, operator)
+            )
+
+        self._render_exprs = render_exprs
         ColumnCollectionConstraint.__init__(
             self,
-            *[col for col, op in elements],
+            *columns,
             name=kw.get('name'),
             deferrable=kw.get('deferrable'),
             initially=kw.get('initially')
-            )
-        self.operators = {}
-        for col_or_string, op in elements:
-            name = getattr(col_or_string, 'name', col_or_string)
-            self.operators[name] = op
+        )
         self.using = kw.get('using', 'gist')
         where = kw.get('where')
-        if where:
-            self.where =  expression._literal_as_text(where)
+        if where is not None:
+            self.where = expression._literal_as_text(where)
 
     def copy(self, **kw):
         elements = [(col, self.operators[col])
                     for col in self.columns.keys()]
         c = self.__class__(*elements,
-                            name=self.name,
-                            deferrable=self.deferrable,
-                            initially=self.initially)
+                           name=self.name,
+                           deferrable=self.deferrable,
+                           initially=self.initially)
         c.dispatch._update(self.dispatch)
         return c
-
