@@ -1,5 +1,6 @@
 # engine/threadlocal.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2017 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -58,7 +59,10 @@ class TLEngine(base.Engine):
             # guards against pool-level reapers, if desired.
             # or not connection.connection.is_valid:
             connection = self._tl_connection_cls(
-                self, self.pool.connect(), **kw)
+                self,
+                self._wrap_pool_connect(
+                    self.pool.connect, connection),
+                **kw)
             self._connections.conn = weakref.ref(connection)
 
         return connection._increment_connect()
@@ -94,20 +98,20 @@ class TLEngine(base.Engine):
 
     def prepare(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         self._connections.trans[-1].prepare()
 
     def commit(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         trans = self._connections.trans.pop(-1)
         trans.commit()
 
     def rollback(self):
         if not hasattr(self._connections, 'trans') or \
-            not self._connections.trans:
+                not self._connections.trans:
             return
         trans = self._connections.trans.pop(-1)
         trans.rollback()
@@ -119,8 +123,8 @@ class TLEngine(base.Engine):
     @property
     def closed(self):
         return not hasattr(self._connections, 'conn') or \
-                self._connections.conn() is None or \
-                self._connections.conn().closed
+            self._connections.conn() is None or \
+            self._connections.conn().closed
 
     def close(self):
         if not self.closed:
