@@ -1,11 +1,11 @@
 import warnings
-from sqlalchemy.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.testing import eq_, is_, assert_raises, assert_raises_message
 from sqlalchemy import *
 from sqlalchemy import exc as sa_exc, util, event
 from sqlalchemy.orm import *
 from sqlalchemy.orm.util import instance_str
 from sqlalchemy.orm import exc as orm_exc, attributes
-from sqlalchemy.testing.assertsql import AllOf, CompiledSQL
+from sqlalchemy.testing.assertsql import AllOf, CompiledSQL, Or
 from sqlalchemy.sql import table, column
 from sqlalchemy import testing
 from sqlalchemy.testing import engines
@@ -28,12 +28,12 @@ class O2MTest(fixtures.MappedTest):
 
         bar = Table('bar', metadata,
             Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-            Column('data', String(20)))
+            Column('bar_data', String(20)))
 
         blub = Table('blub', metadata,
             Column('id', Integer, ForeignKey('bar.id'), primary_key=True),
             Column('foo_id', Integer, ForeignKey('foo.id'), nullable=False),
-            Column('data', String(20)))
+            Column('blub_data', String(20)))
 
     def test_basic(self):
         class Foo(object):
@@ -77,6 +77,65 @@ class O2MTest(fixtures.MappedTest):
         eq_(l[0].parent_foo.data, 'foo #1')
         eq_(l[1].parent_foo.data, 'foo #1')
 
+
+class PolyExpressionEagerLoad(fixtures.DeclarativeMappedTest):
+    run_setup_mappers = 'once'
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+
+        class A(fixtures.ComparableEntity, Base):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True,
+                        test_needs_autoincrement=True)
+            discriminator = Column(String(50), nullable=False)
+            child_id = Column(Integer, ForeignKey('a.id'))
+            child = relationship('A')
+
+            p_a = case([
+                (discriminator == "a", "a"),
+            ], else_="b")
+
+            __mapper_args__ = {
+                'polymorphic_identity': 'a',
+                "polymorphic_on": p_a,
+            }
+
+        class B(A):
+            __mapper_args__ = {
+                'polymorphic_identity': 'b'
+            }
+
+    @classmethod
+    def insert_data(cls):
+        A = cls.classes.A
+
+        session = Session(testing.db)
+        session.add_all([
+            A(id=1, discriminator='a'),
+            A(id=2, discriminator='b', child_id=1),
+            A(id=3, discriminator='c', child_id=1),
+        ])
+        session.commit()
+
+    def test_joinedload(self):
+        A = self.classes.A
+        B = self.classes.B
+
+        session = Session(testing.db)
+        result = session.query(A).filter_by(child_id=None).\
+            options(joinedload('child')).one()
+
+
+        eq_(
+            result,
+            A(id=1, discriminator='a', child=[B(id=2), B(id=3)]),
+        )
+
+
 class PolymorphicResolutionMultiLevel(fixtures.DeclarativeMappedTest,
                                         testing.AssertsCompiledSQL):
     run_setup_mappers = 'once'
@@ -184,7 +243,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
                 Column('x', String(10)),
                 Column('q', String(10)))
         t2 = Table('t2', metadata,
-                Column('id', Integer, primary_key=True,
+                Column('t2id', Integer, primary_key=True,
                             test_needs_autoincrement=True),
                 Column('y', String(10)),
                 Column('xid', ForeignKey('t1.id')))
@@ -396,6 +455,22 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
     def _roundtrip(self, set_event=True, parent_ident='parent', child_ident='child'):
         Parent, Child = self.classes.Parent, self.classes.Child
 
+        # locate the "polymorphic_on" ColumnProperty.   This isn't
+        # "officially" stored at the moment so do some heuristics to find it.
+        parent_mapper = inspect(Parent)
+        for prop in parent_mapper.column_attrs:
+            if not prop.instrument:
+                break
+        else:
+            prop = parent_mapper._columntoproperty[
+                parent_mapper.polymorphic_on]
+
+        # then make sure the column we will query on matches.
+        is_(
+            parent_mapper.polymorphic_on,
+            prop.columns[0]
+        )
+
         if set_event:
             @event.listens_for(Parent, "init", propagate=True)
             def set_identity(instance, *arg, **kw):
@@ -583,7 +658,8 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
                         polymorphic_identity='a')
         mapper(B, table_b, inherits=A,
                         polymorphic_on=table_b.c.class_name,
-                        polymorphic_identity='b')
+                        polymorphic_identity='b',
+                        properties=dict(class_name=[table_a.c.class_name, table_b.c.class_name]))
         mapper(C, table_c, inherits=B,
                         polymorphic_identity='c')
         mapper(D, inherits=B,
@@ -854,13 +930,13 @@ class GetTest(fixtures.MappedTest):
 
         bar = Table('bar', metadata,
             Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-            Column('data', String(20)))
+            Column('bar_data', String(20)))
 
         blub = Table('blub', metadata,
-            Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('blub_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('foo_id', Integer, ForeignKey('foo.id')),
             Column('bar_id', Integer, ForeignKey('bar.id')),
-            Column('data', String(20)))
+            Column('blub_data', String(20)))
 
     @classmethod
     def setup_classes(cls):
@@ -957,14 +1033,13 @@ class EagerLazyTest(fixtures.MappedTest):
                     Column('data', String(30)))
         bar = Table('bar', metadata,
                     Column('id', Integer, ForeignKey('foo.id'), primary_key=True),
-                    Column('data', String(30)))
+                    Column('bar_data', String(30)))
 
         bar_foo = Table('bar_foo', metadata,
                         Column('bar_id', Integer, ForeignKey('bar.id')),
                         Column('foo_id', Integer, ForeignKey('foo.id'))
         )
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def test_basic(self):
         class Foo(object): pass
         class Bar(Foo): pass
@@ -1148,6 +1223,62 @@ class FlushTest(fixtures.MappedTest):
         sess.flush()
         assert user_roles.count().scalar() == 1
 
+
+class OptimizedGetOnDeferredTest(fixtures.MappedTest):
+    """test that the 'optimized get' path accommodates deferred columns."""
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table(
+            "a", metadata,
+            Column('id', Integer, primary_key=True,
+                   test_needs_autoincrement=True)
+        )
+        Table(
+            "b", metadata,
+            Column('id', Integer, ForeignKey('a.id'), primary_key=True),
+            Column('data', String(10))
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.Basic):
+            pass
+
+        class B(A):
+            pass
+
+    @classmethod
+    def setup_mappers(cls):
+        A, B = cls.classes("A", "B")
+        a, b = cls.tables("a", "b")
+
+        mapper(A, a)
+        mapper(B, b, inherits=A, properties={
+            'data': deferred(b.c.data),
+            'expr': column_property(b.c.data + 'q', deferred=True)
+        })
+
+    def test_column_property(self):
+        A, B = self.classes("A", "B")
+        sess = Session()
+        b1 = B(data='x')
+        sess.add(b1)
+        sess.flush()
+
+        eq_(b1.expr, 'xq')
+
+    def test_expired_column(self):
+        A, B = self.classes("A", "B")
+        sess = Session()
+        b1 = B(data='x')
+        sess.add(b1)
+        sess.flush()
+        sess.expire(b1, ['data'])
+
+        eq_(b1.data, 'x')
+
+
 class JoinedNoFKSortingTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -1176,9 +1307,11 @@ class JoinedNoFKSortingTest(fixtures.MappedTest):
         A, B, C = cls.classes.A, cls.classes.B, cls.classes.C
         mapper(A, cls.tables.a)
         mapper(B, cls.tables.b, inherits=A,
-                    inherit_condition=cls.tables.a.c.id == cls.tables.b.c.id)
+                    inherit_condition=cls.tables.a.c.id == cls.tables.b.c.id,
+                    inherit_foreign_keys=cls.tables.b.c.id)
         mapper(C, cls.tables.c, inherits=A,
-                    inherit_condition=cls.tables.a.c.id == cls.tables.c.c.id)
+                    inherit_condition=cls.tables.a.c.id == cls.tables.c.c.id,
+                    inherit_foreign_keys=cls.tables.c.c.id)
 
     def test_ordering(self):
         B, C = self.classes.B, self.classes.C
@@ -1346,7 +1479,7 @@ class DistinctPKTest(fixtures.MappedTest):
                 )
 
         employee_table = Table("employees", metadata,
-                Column("id", Integer, primary_key=True, test_needs_autoincrement=True),
+                Column("eid", Integer, primary_key=True, test_needs_autoincrement=True),
                 Column("salary", Integer),
                 Column("person_id", Integer, ForeignKey("persons.id")),
                 )
@@ -1376,18 +1509,19 @@ class DistinctPKTest(fixtures.MappedTest):
         person_mapper = mapper(Person, person_table)
         mapper(Employee, employee_table, inherits=person_mapper,
                         properties={'pid':person_table.c.id,
-                                    'eid':employee_table.c.id})
+                                    'eid':employee_table.c.eid})
         self._do_test(False)
 
     def test_explicit_composite_pk(self):
         person_mapper = mapper(Person, person_table)
         mapper(Employee, employee_table,
                     inherits=person_mapper,
-                    primary_key=[person_table.c.id, employee_table.c.id])
+                    properties=dict(id=[employee_table.c.eid, person_table.c.id]),
+                    primary_key=[person_table.c.id, employee_table.c.eid])
         assert_raises_message(sa_exc.SAWarning,
                                     r"On mapper Mapper\|Employee\|employees, "
                                     "primary key column 'persons.id' is being "
-                                    "combined with distinct primary key column 'employees.id' "
+                                    "combined with distinct primary key column 'employees.eid' "
                                     "in attribute 'id'.  Use explicit properties to give "
                                     "each column its own mapped attribute name.",
             self._do_test, True
@@ -1448,7 +1582,7 @@ class SyncCompileTest(fixtures.MappedTest):
     def _do_test(self, j1, j2):
         class A(object):
            def __init__(self, **kwargs):
-               for key, value in kwargs.items():
+               for key, value in list(kwargs.items()):
                     setattr(self, key, value)
 
         class B(A):
@@ -1488,7 +1622,7 @@ class OverrideColKeyTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        global base, subtable
+        global base, subtable, subtable_two
 
         base = Table('base', metadata,
             Column('base_id', Integer, primary_key=True, test_needs_autoincrement=True),
@@ -1500,6 +1634,12 @@ class OverrideColKeyTest(fixtures.MappedTest):
             Column('base_id', Integer, ForeignKey('base.base_id'), primary_key=True),
             Column('subdata', String(255))
         )
+        subtable_two = Table('subtable_two', metadata,
+            Column('base_id', Integer, primary_key=True),
+            Column('fk_base_id', Integer, ForeignKey('base.base_id')),
+            Column('subdata', String(255))
+        )
+
 
     def test_plain(self):
         # control case
@@ -1610,6 +1750,23 @@ class OverrideColKeyTest(fixtures.MappedTest):
         # an exception in 0.7 due to the implicit conflict.
         assert_raises(sa_exc.InvalidRequestError, go)
 
+    def test_pk_fk_different(self):
+        class Base(object):
+            pass
+        class Sub(Base):
+            pass
+
+        mapper(Base, base)
+
+        def go():
+            mapper(Sub, subtable_two, inherits=Base)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Implicitly combining column base.base_id with "
+            "column subtable_two.base_id under attribute 'base_id'",
+            go
+        )
+
     def test_plain_descriptor(self):
         """test that descriptors prevent inheritance from propigating properties to subclasses."""
 
@@ -1717,12 +1874,12 @@ class OptimizedLoadTest(fixtures.MappedTest):
         Table('sub', metadata,
             Column('id', Integer, ForeignKey('base.id'), primary_key=True),
             Column('sub', String(50)),
-            Column('counter', Integer, server_default="1"),
-            Column('counter2', Integer, server_default="1")
+            Column('subcounter', Integer, server_default="1"),
+            Column('subcounter2', Integer, server_default="1")
         )
         Table('subsub', metadata,
             Column('id', Integer, ForeignKey('sub.id'), primary_key=True),
-            Column('counter2', Integer, server_default="1")
+            Column('subsubcounter2', Integer, server_default="1")
         )
         Table('with_comp', metadata,
             Column('id', Integer, ForeignKey('base.id'), primary_key=True),
@@ -1744,7 +1901,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
         mapper(Base, base)
         mapper(JoinBase, base.outerjoin(sub), properties=util.OrderedDict(
                 [('id', [base.c.id, sub.c.id]),
-                ('counter', [base.c.counter, sub.c.counter])])
+                ('counter', [base.c.counter, sub.c.subcounter])])
             )
         mapper(SubJoinBase, inherits=JoinBase)
 
@@ -1766,11 +1923,10 @@ class OptimizedLoadTest(fixtures.MappedTest):
             go,
             CompiledSQL(
                 "SELECT base.id AS base_id, sub.id AS sub_id, "
-                "base.counter AS base_counter, sub.counter AS sub_counter, "
-                "base.data AS base_data, "
-                "base.type AS base_type, sub.sub AS sub_sub, "
-                "sub.counter2 AS sub_counter2 FROM base "
-                "LEFT OUTER JOIN sub ON base.id = sub.id "
+                "base.counter AS base_counter, sub.subcounter AS sub_subcounter, "
+                "base.data AS base_data, base.type AS base_type, "
+                "sub.sub AS sub_sub, sub.subcounter2 AS sub_subcounter2 "
+                "FROM base LEFT OUTER JOIN sub ON base.id = sub.id "
                 "WHERE base.id = :param_1",
                 {'param_1': sjb_id}
             ),
@@ -1915,15 +2071,15 @@ class OptimizedLoadTest(fixtures.MappedTest):
                 ),
         )
         def go():
-            eq_( s1.counter2, 1 )
+            eq_( s1.subcounter2, 1 )
         self.assert_sql_execution(
             testing.db,
             go,
             CompiledSQL(
-                "SELECT sub.counter AS sub_counter, base.counter AS base_counter, "
-                "sub.counter2 AS sub_counter2 FROM base JOIN sub ON "
-                "base.id = sub.id WHERE base.id = :param_1",
-                lambda ctx:{u'param_1': s1.id}
+                "SELECT base.counter AS base_counter, sub.subcounter AS sub_subcounter, "
+                "sub.subcounter2 AS sub_subcounter2 FROM base JOIN sub "
+                "ON base.id = sub.id WHERE base.id = :param_1",
+                lambda ctx:{'param_1': s1.id}
             ),
         )
 
@@ -1940,19 +2096,19 @@ class OptimizedLoadTest(fixtures.MappedTest):
 
         s1 = Sub()
         assert m._optimized_get_statement(attributes.instance_state(s1),
-                                ['counter2']) is None
+                                ['subcounter2']) is None
 
         # loads s1.id as None
         eq_(s1.id, None)
 
         # this now will come up with a value of None for id - should reject
         assert m._optimized_get_statement(attributes.instance_state(s1),
-                                ['counter2']) is None
+                                ['subcounter2']) is None
 
         s1.id = 1
         attributes.instance_state(s1)._commit_all(s1.__dict__, None)
         assert m._optimized_get_statement(attributes.instance_state(s1),
-                                ['counter2']) is not None
+                                ['subcounter2']) is not None
 
     def test_load_expired_on_pending_twolevel(self):
         base, sub, subsub = (self.tables.base,
@@ -1971,7 +2127,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
         mapper(Sub, sub, inherits=Base, polymorphic_identity='sub')
         mapper(SubSub, subsub, inherits=Sub, polymorphic_identity='subsub')
         sess = Session()
-        s1 = SubSub(data='s1', counter=1)
+        s1 = SubSub(data='s1', counter=1, subcounter=2)
         sess.add(s1)
         self.assert_sql_execution(
                 testing.db,
@@ -1982,9 +2138,9 @@ class OptimizedLoadTest(fixtures.MappedTest):
                     [{'data':'s1','type':'subsub','counter':1}]
                 ),
                 CompiledSQL(
-                    "INSERT INTO sub (id, sub, counter) VALUES "
-                    "(:id, :sub, :counter)",
-                    lambda ctx:[{'counter': 1, 'sub': None, 'id': s1.id}]
+                    "INSERT INTO sub (id, sub, subcounter) VALUES "
+                    "(:id, :sub, :subcounter)",
+                    lambda ctx:[{'subcounter': 2, 'sub': None, 'id': s1.id}]
                 ),
                 CompiledSQL(
                     "INSERT INTO subsub (id) VALUES (:id)",
@@ -1994,78 +2150,28 @@ class OptimizedLoadTest(fixtures.MappedTest):
 
         def go():
             eq_(
-                s1.counter2, 1
+                s1.subcounter2, 1
             )
         self.assert_sql_execution(
             testing.db,
             go,
-            CompiledSQL(
-                "SELECT subsub.counter2 AS subsub_counter2, "
-                "sub.counter2 AS sub_counter2 FROM subsub, sub "
-                "WHERE :param_1 = sub.id AND sub.id = subsub.id",
-                lambda ctx:{u'param_1': s1.id}
-            ),
+            Or(
+                CompiledSQL(
+                    "SELECT subsub.subsubcounter2 AS subsub_subsubcounter2, "
+                    "sub.subcounter2 AS sub_subcounter2 FROM subsub, sub "
+                    "WHERE :param_1 = sub.id AND sub.id = subsub.id",
+                    lambda ctx: {'param_1': s1.id}
+                ),
+                CompiledSQL(
+                    "SELECT sub.subcounter2 AS sub_subcounter2, "
+                    "subsub.subsubcounter2 AS subsub_subsubcounter2 "
+                    "FROM sub, subsub "
+                    "WHERE :param_1 = sub.id AND sub.id = subsub.id",
+                    lambda ctx: {'param_1': s1.id}
+                ),
+            )
         )
 
-class TransientInheritingGCTest(fixtures.TestBase):
-    __requires__ = ('cpython',)
-
-    def _fixture(self):
-        Base = declarative_base()
-
-        class A(Base):
-            __tablename__ = 'a'
-            id = Column(Integer, primary_key=True,
-                                    test_needs_autoincrement=True)
-            data = Column(String(10))
-        self.A = A
-        return Base
-
-    def setUp(self):
-        self.Base = self._fixture()
-
-    def tearDown(self):
-        self.Base.metadata.drop_all(testing.db)
-        #clear_mappers()
-        self.Base = None
-
-    def _do_test(self, go):
-        B = go()
-        self.Base.metadata.create_all(testing.db)
-        sess = Session(testing.db)
-        sess.add(B(data='some b'))
-        sess.commit()
-
-        b1 = sess.query(B).one()
-        assert isinstance(b1, B)
-        sess.close()
-        del sess
-        del b1
-        del B
-
-        gc_collect()
-
-        eq_(
-            len(self.A.__subclasses__()),
-            0)
-
-    def test_single(self):
-        def go():
-            class B(self.A):
-                pass
-            return B
-        self._do_test(go)
-
-    @testing.fails_if(lambda: True,
-                "not supported for joined inh right now.")
-    def test_joined(self):
-        def go():
-            class B(self.A):
-                __tablename__ = 'b'
-                id = Column(Integer, ForeignKey('a.id'),
-                        primary_key=True)
-            return B
-        self._do_test(go)
 
 class NoPKOnSubTableWarningTest(fixtures.TestBase):
 
@@ -2236,7 +2342,8 @@ class InhCondTest(fixtures.TestBase):
 
         assert_raises_message(
             sa_exc.NoReferencedColumnError,
-            "Could not create ForeignKey 'base.q' on table "
+            "Could not initialize target column for ForeignKey "
+            "'base.q' on table "
             "'derived': table 'base' has no column named 'q'",
             mapper,
             Derived, derived_table,  inherits=Base
@@ -2502,9 +2609,9 @@ class NameConflictTest(fixtures.MappedTest):
                     polymorphic_identity='foo')
         sess = create_session()
         f = Foo()
-        f.content_type = u'bar'
+        f.content_type = 'bar'
         sess.add(f)
         sess.flush()
         f_id = f.id
         sess.expunge_all()
-        assert sess.query(Content).get(f_id).content_type == u'bar'
+        assert sess.query(Content).get(f_id).content_type == 'bar'
