@@ -1,6 +1,6 @@
 from sqlalchemy.orm import create_session, relationship, mapper, \
     contains_eager, joinedload, subqueryload, subqueryload_all,\
-    Session, aliased, with_polymorphic, joinedload_all
+    Session, aliased, with_polymorphic, joinedload_all, backref
 
 from sqlalchemy import Integer, String, ForeignKey
 from sqlalchemy.engine import default
@@ -1361,10 +1361,70 @@ class SubClassToSubClassMultiTest(AssertsCompiledSQL, fixtures.MappedTest):
         )
 
 
+class JoinedloadSinglePolysubSingle(
+        fixtures.DeclarativeMappedTest,
+        testing.AssertsCompiledSQL):
+    """exercise issue #3611, using the test from dupe issue 3614"""
+
+    run_define_tables = None
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+
+        class User(Base):
+            __tablename__ = 'users'
+            id = Column(Integer, primary_key=True)
+
+        class UserRole(Base):
+            __tablename__ = 'user_roles'
+
+            id = Column(Integer, primary_key=True)
+
+            row_type = Column(String(50), nullable=False)
+            __mapper_args__ = {'polymorphic_on': row_type}
+
+            user_id = Column(Integer, ForeignKey('users.id'), nullable=False)
+            user = relationship('User', lazy=False)
+
+        class Admin(UserRole):
+            __tablename__ = 'admins'
+            __mapper_args__ = {'polymorphic_identity': 'admin'}
+
+            id = Column(Integer, ForeignKey('user_roles.id'), primary_key=True)
+
+        class Thing(Base):
+            __tablename__ = 'things'
+
+            id = Column(Integer, primary_key=True)
+
+            admin_id = Column(Integer, ForeignKey('admins.id'))
+            admin = relationship('Admin', lazy=False)
+
+    def test_query(self):
+        Thing = self.classes.Thing
+        sess = Session()
+        self.assert_compile(
+            sess.query(Thing),
+            "SELECT things.id AS things_id, "
+            "things.admin_id AS things_admin_id, "
+            "users_1.id AS users_1_id, admins_1.id AS admins_1_id, "
+            "user_roles_1.id AS user_roles_1_id, "
+            "user_roles_1.row_type AS user_roles_1_row_type, "
+            "user_roles_1.user_id AS user_roles_1_user_id FROM things "
+            "LEFT OUTER JOIN (user_roles AS user_roles_1 JOIN admins "
+            "AS admins_1 ON user_roles_1.id = admins_1.id) ON "
+            "admins_1.id = things.admin_id "
+            "LEFT OUTER JOIN users AS "
+            "users_1 ON users_1.id = user_roles_1.user_id"
+        )
+
+
 class JoinedloadOverWPolyAliased(
         fixtures.DeclarativeMappedTest,
         testing.AssertsCompiledSQL):
-    """exercise issues in #3593"""
+    """exercise issues in #3593 and #3611"""
 
     run_setup_mappers = 'each'
     run_setup_classes = 'each'
@@ -1375,6 +1435,20 @@ class JoinedloadOverWPolyAliased(
     def setup_classes(cls):
         Base = cls.DeclarativeBasic
 
+        class Owner(Base):
+            __tablename__ = 'owner'
+
+            id = Column(Integer, primary_key=True)
+            type = Column(String(20))
+
+            __mapper_args__ = {
+                'polymorphic_on': type,
+                'with_polymorphic': ('*', None),
+            }
+
+        class SubOwner(Owner):
+            __mapper_args__ = {'polymorphic_identity': 'so'}
+
         class Parent(Base):
             __tablename__ = 'parent'
 
@@ -1397,14 +1471,11 @@ class JoinedloadOverWPolyAliased(
             child_id = Column(
                 Integer, ForeignKey('parent.id'), primary_key=True)
 
-            child = relationship(
-                Parent,
-                primaryjoin=child_id == Parent.id,
-            )
-
     def _fixture_from_base(self):
         Parent = self.classes.Parent
         Link = self.classes.Link
+        Link.child = relationship(
+            Parent, primaryjoin=Link.child_id == Parent.id)
 
         Parent.links = relationship(
             Link,
@@ -1415,6 +1486,9 @@ class JoinedloadOverWPolyAliased(
     def _fixture_from_subclass(self):
         Sub1 = self.classes.Sub1
         Link = self.classes.Link
+        Parent = self.classes.Parent
+        Link.child = relationship(
+            Parent, primaryjoin=Link.child_id == Parent.id)
 
         Sub1.links = relationship(
             Link,
@@ -1422,13 +1496,50 @@ class JoinedloadOverWPolyAliased(
         )
         return Sub1
 
+    def _fixture_to_subclass_to_base(self):
+        Owner = self.classes.Owner
+        Parent = self.classes.Parent
+        Sub1 = self.classes.Sub1
+        Link = self.classes.Link
+
+        # Link -> Sub1 -> Owner
+
+        Link.child = relationship(
+            Sub1, primaryjoin=Link.child_id == Sub1.id)
+
+        Parent.owner_id = Column(ForeignKey('owner.id'))
+
+        Parent.owner = relationship(Owner)
+        return Parent
+
+    def _fixture_to_base_to_base(self):
+        Owner = self.classes.Owner
+        Parent = self.classes.Parent
+        Link = self.classes.Link
+
+        # Link -> Parent -> Owner
+
+        Link.child = relationship(
+            Parent, primaryjoin=Link.child_id == Parent.id)
+
+        Parent.owner_id = Column(ForeignKey('owner.id'))
+
+        Parent.owner = relationship(Owner)
+        return Parent
+
     def test_from_base(self):
-        self._test(self._fixture_from_base)
+        self._test_poly_single_poly(self._fixture_from_base)
 
     def test_from_sub(self):
-        self._test(self._fixture_from_subclass)
+        self._test_poly_single_poly(self._fixture_from_subclass)
 
-    def _test(self, fn):
+    def test_to_sub_to_base(self):
+        self._test_single_poly_poly(self._fixture_to_subclass_to_base)
+
+    def test_to_base_to_base(self):
+        self._test_single_poly_poly(self._fixture_to_base_to_base)
+
+    def _test_poly_single_poly(self, fn):
         cls = fn()
         Link = self.classes.Link
 
@@ -1448,17 +1559,47 @@ class JoinedloadOverWPolyAliased(
         self.assert_compile(
             q,
             "SELECT parent.id AS parent_id, parent.type AS parent_type, "
-            "parent_1.id AS parent_1_id, parent_1.type AS parent_1_type, "
             "link_1.parent_id AS link_1_parent_id, "
             "link_1.child_id AS link_1_child_id, "
+            "parent_1.id AS parent_1_id, parent_1.type AS parent_1_type, "
             "link_2.parent_id AS link_2_parent_id, "
             "link_2.child_id AS link_2_child_id "
             "FROM parent "
-            "LEFT OUTER JOIN link AS link_2 ON parent.id = link_2.parent_id "
+            "LEFT OUTER JOIN link AS link_1 ON parent.id = link_1.parent_id "
             "LEFT OUTER JOIN parent "
-            "AS parent_1 ON link_2.child_id = parent_1.id "
-            "LEFT OUTER JOIN link AS link_1 "
-            "ON parent_1.id = link_1.parent_id" + extra
+            "AS parent_1 ON link_1.child_id = parent_1.id "
+            "LEFT OUTER JOIN link AS link_2 "
+            "ON parent_1.id = link_2.parent_id" + extra
+        )
+
+    def _test_single_poly_poly(self, fn):
+        parent_cls = fn()
+        Link = self.classes.Link
+
+        session = Session()
+        q = session.query(Link).options(
+            joinedload_all(
+                Link.child,
+                parent_cls.owner
+            )
+        )
+
+        if Link.child.property.mapper.class_ is self.classes.Sub1:
+            extra = "AND parent_1.type IN (:type_1) "
+        else:
+            extra = ""
+
+        self.assert_compile(
+            q,
+            "SELECT link.parent_id AS link_parent_id, "
+            "link.child_id AS link_child_id, parent_1.id AS parent_1_id, "
+            "parent_1.type AS parent_1_type, "
+            "parent_1.owner_id AS parent_1_owner_id, "
+            "owner_1.id AS owner_1_id, owner_1.type AS owner_1_type "
+            "FROM link LEFT OUTER JOIN parent AS parent_1 "
+            "ON link.child_id = parent_1.id "  + extra +
+            "LEFT OUTER JOIN owner AS owner_1 "
+            "ON owner_1.id = parent_1.owner_id"
         )
 
     def test_local_wpoly(self):
@@ -1477,15 +1618,15 @@ class JoinedloadOverWPolyAliased(
         self.assert_compile(
             q,
             "SELECT parent.id AS parent_id, parent.type AS parent_type, "
-            "parent_1.id AS parent_1_id, parent_1.type AS parent_1_type, "
             "link_1.parent_id AS link_1_parent_id, "
             "link_1.child_id AS link_1_child_id, "
+            "parent_1.id AS parent_1_id, parent_1.type AS parent_1_type, "
             "link_2.parent_id AS link_2_parent_id, "
             "link_2.child_id AS link_2_child_id FROM parent "
-            "LEFT OUTER JOIN link AS link_2 ON parent.id = link_2.parent_id "
+            "LEFT OUTER JOIN link AS link_1 ON parent.id = link_1.parent_id "
             "LEFT OUTER JOIN parent AS parent_1 "
-            "ON link_2.child_id = parent_1.id "
-            "LEFT OUTER JOIN link AS link_1 ON parent_1.id = link_1.parent_id"
+            "ON link_1.child_id = parent_1.id "
+            "LEFT OUTER JOIN link AS link_2 ON parent_1.id = link_2.parent_id"
         )
 
 
@@ -1710,4 +1851,73 @@ class MultipleAdaptUsesEntityOverTableTest(AssertsCompiledSQL, fixtures.MappedTe
             "(a AS a_1 JOIN c AS c_1 ON a_1.id = c_1.id) ON c_1.bid = b.id "
             "JOIN (a AS a_2 JOIN d AS d_1 ON a_2.id = d_1.id) "
             "ON d_1.cid = c_1.id"
-        )
\ No newline at end of file
+        )
+
+
+class BetweenSubclassJoinWExtraJoinedLoad(
+        fixtures.DeclarativeMappedTest,
+        testing.AssertsCompiledSQL):
+    """test for [ticket:3884]"""
+
+    run_define_tables = None
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+
+        class Person(Base):
+            __tablename__ = 'people'
+            id = Column(Integer, primary_key=True)
+            discriminator = Column('type', String(50))
+            __mapper_args__ = {'polymorphic_on': discriminator}
+
+        class Manager(Person):
+            __tablename__ = 'managers'
+            __mapper_args__ = {'polymorphic_identity': 'manager'}
+            id = Column(Integer, ForeignKey('people.id'), primary_key=True)
+
+        class Engineer(Person):
+            __tablename__ = 'engineers'
+            __mapper_args__ = {'polymorphic_identity': 'engineer'}
+            id = Column(Integer, ForeignKey('people.id'), primary_key=True)
+            primary_language = Column(String(50))
+            manager_id = Column(Integer, ForeignKey('managers.id'))
+            manager = relationship(
+                Manager, primaryjoin=(Manager.id == manager_id))
+
+        class LastSeen(Base):
+            __tablename__ = 'seen'
+            id = Column(Integer, ForeignKey('people.id'), primary_key=True)
+            timestamp = Column(Integer)
+            taggable = relationship(
+                Person, primaryjoin=(Person.id == id),
+                backref=backref("last_seen", lazy=False))
+
+    def test_query(self):
+        Engineer, Manager = self.classes("Engineer", "Manager")
+
+        sess = Session()
+
+        # eager join is both from Enginer->LastSeen as well as
+        # Manager->LastSeen.  In the case of Manager->LastSeen,
+        # Manager is internally aliased, and comes to JoinedEagerLoader
+        # with no "parent" entity but an adapter.
+        q = sess.query(Engineer, Manager).join(Engineer.manager)
+        self.assert_compile(
+            q,
+            "SELECT people.type AS people_type, engineers.id AS engineers_id, "
+            "people.id AS people_id, "
+            "engineers.primary_language AS engineers_primary_language, "
+            "engineers.manager_id AS engineers_manager_id, "
+            "people_1.type AS people_1_type, managers_1.id AS managers_1_id, "
+            "people_1.id AS people_1_id, seen_1.id AS seen_1_id, "
+            "seen_1.timestamp AS seen_1_timestamp, seen_2.id AS seen_2_id, "
+            "seen_2.timestamp AS seen_2_timestamp "
+            "FROM people JOIN engineers ON people.id = engineers.id "
+            "JOIN (people AS people_1 JOIN managers AS managers_1 "
+            "ON people_1.id = managers_1.id) "
+            "ON managers_1.id = engineers.manager_id LEFT OUTER JOIN "
+            "seen AS seen_1 ON people.id = seen_1.id LEFT OUTER JOIN "
+            "seen AS seen_2 ON people_1.id = seen_2.id"
+        )
