@@ -1,18 +1,14 @@
 from sqlalchemy.testing import assert_raises, assert_raises_message
 import sqlalchemy as sa
 from sqlalchemy import testing
-from sqlalchemy import MetaData, Integer, String, ForeignKey, func, \
-    util, select
+from sqlalchemy import Integer, String, ForeignKey, \
+    select
 from sqlalchemy.testing.schema import Table, Column
-from sqlalchemy.orm import mapper, relationship, backref, \
-    class_mapper, CompositeProperty, \
-    validates, aliased, configure_mappers
-from sqlalchemy.orm import attributes, \
-    composite, relationship, \
-    Session
+from sqlalchemy.orm import mapper, relationship, \
+    CompositeProperty, aliased
+from sqlalchemy.orm import composite, Session, configure_mappers
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
-from test.orm import _fixtures
 
 
 class PointTest(fixtures.MappedTest):
@@ -82,6 +78,12 @@ class PointTest(fixtures.MappedTest):
         sess.commit()
         return sess
 
+    def test_early_configure(self):
+        # test [ticket:2935], that we can call a composite
+        # expression before configure_mappers()
+        Edge = self.classes.Edge
+        Edge.start.__clause_element__()
+
     def test_round_trip(self):
         Graph, Point = self.classes.Graph, self.classes.Point
 
@@ -214,17 +216,45 @@ class PointTest(fixtures.MappedTest):
             ((), [Point(x=None, y=None)], ())
         )
 
-    def test_query_cols(self):
+    def test_query_cols_legacy(self):
         Edge = self.classes.Edge
 
         sess = self._fixture()
 
         eq_(
-            sess.query(Edge.start, Edge.end).all(),
+            sess.query(Edge.start.clauses, Edge.end.clauses).all(),
             [(3, 4, 5, 6), (14, 5, 2, 7)]
         )
 
+    def test_query_cols(self):
+        Edge = self.classes.Edge
+        Point = self.classes.Point
+
+        sess = self._fixture()
+
+        start, end = Edge.start, Edge.end
+
+        eq_(
+            sess.query(start, end).filter(start == Point(3, 4)).all(),
+            [(Point(3, 4), Point(5, 6))]
+        )
+
+    def test_query_cols_labeled(self):
+        Edge = self.classes.Edge
+        Point = self.classes.Point
+
+        sess = self._fixture()
+
+        start, end = Edge.start, Edge.end
+
+        row = sess.query(start.label('s1'), end).filter(start == Point(3, 4)).first()
+        eq_(row.s1.x, 3)
+        eq_(row.s1.y, 4)
+        eq_(row.end.x, 5)
+        eq_(row.end.y, 6)
+
     def test_delete(self):
+        Point = self.classes.Point
         Graph, Edge = self.classes.Graph, self.classes.Edge
 
         sess = self._fixture()
@@ -235,7 +265,10 @@ class PointTest(fixtures.MappedTest):
         sess.flush()
         eq_(
             sess.query(Edge.start, Edge.end).all(),
-            [(3, 4, 5, 6), (14, 5, None, None)]
+            [
+                (Point(x=3, y=4), Point(x=5, y=6)),
+                (Point(x=14, y=5), Point(x=None, y=None))
+            ]
         )
 
     def test_save_null(self):
@@ -523,11 +556,11 @@ class MappedSelectTest(fixtures.MappedTest):
         session.commit()
         eq_(
             testing.db.execute(descriptions.select()).fetchall(),
-            [(1, u'Color', u'Number')]
+            [(1, 'Color', 'Number')]
         )
         eq_(
             testing.db.execute(values.select()).fetchall(),
-            [(1, 1, u'Red', u'5'), (2, 1, u'Blue', u'1')]
+            [(1, 1, 'Red', '5'), (2, 1, 'Blue', '1')]
         )
 
 class ManyToOneTest(fixtures.MappedTest):
@@ -575,6 +608,13 @@ class ManyToOneTest(fixtures.MappedTest):
         })
         mapper(B, b)
 
+    def test_early_configure(self):
+        # test [ticket:2935], that we can call a composite
+        # expression before configure_mappers()
+        A = self.classes.A
+        A.c.__clause_element__()
+
+
     def test_persist(self):
         A, C, B = (self.classes.A,
                                 self.classes.C,
@@ -725,8 +765,8 @@ class ConfigurationTest(fixtures.MappedTest):
             # renders here, so the "%" operator in the string needs to
             # apply the tuple also
             r"Composite expects Column objects or mapped "
-            "attributes/attribute names as "
-            "arguments, got: \(Column",
+            r"attributes/attribute names as "
+            r"arguments, got: \(Column",
             configure_mappers
         )
 
@@ -881,3 +921,15 @@ class ComparatorTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
             "edge_1.x2, edge_1.y2"
         )
 
+    def test_clause_expansion(self):
+        self._fixture(False)
+        Edge = self.classes.Edge
+        from sqlalchemy.orm import configure_mappers
+        configure_mappers()
+
+        self.assert_compile(
+            select([Edge]).order_by(Edge.start),
+            "SELECT edge.id, edge.x1, edge.y1, edge.x2, edge.y2 FROM edge "
+            "ORDER BY edge.x1, edge.y1"
+        )
+
