@@ -71,7 +71,9 @@ class TestORMInspection(_fixtures.FixtureTest):
         user_table = self.tables.users
         addresses_table = self.tables.addresses
         mapper(Foo, user_table, with_polymorphic=(Bar,))
-        mapper(Bar, addresses_table, inherits=Foo)
+        mapper(Bar, addresses_table, inherits=Foo, properties={
+                'address_id': addresses_table.c.id
+            })
         i1 = inspect(Foo)
         i2 = inspect(Foo)
         assert i1.selectable is i2.selectable
@@ -339,10 +341,10 @@ class TestORMInspection(_fixtures.FixtureTest):
             insp.attrs.id.value,
             None
         )
-        # now the None is there
+        # nope, still not set
         eq_(
             insp.attrs.id.loaded_value,
-            None
+            NO_VALUE
         )
 
     def test_instance_state_attr_passive_value_collection(self):
@@ -421,6 +423,34 @@ class TestORMInspection(_fixtures.FixtureTest):
         )
         assert 'name' in u1.__dict__
 
+    def test_attrs_props_prop_added_after_configure(self):
+        class AnonClass(object):
+            pass
+
+        from sqlalchemy.orm import mapper, column_property
+        from sqlalchemy.ext.hybrid import hybrid_property
+        m = mapper(AnonClass, self.tables.users)
+
+        eq_(
+            set(inspect(AnonClass).attrs.keys()),
+            set(['id', 'name']))
+        eq_(
+            set(inspect(AnonClass).all_orm_descriptors.keys()),
+            set(['id', 'name']))
+
+        m.add_property('q', column_property(self.tables.users.c.name))
+
+        def desc(self):
+            return self.name
+        AnonClass.foob = hybrid_property(desc)
+
+        eq_(
+            set(inspect(AnonClass).attrs.keys()),
+            set(['id', 'name', 'q']))
+        eq_(
+            set(inspect(AnonClass).all_orm_descriptors.keys()),
+            set(['id', 'name', 'q', 'foob']))
+
     def test_instance_state_ident_transient(self):
         User = self.classes.User
         u1 = User(name='ed')
