@@ -1,15 +1,13 @@
 from __future__ import with_statement
-from sqlalchemy.testing import eq_, assert_raises, \
-    assert_raises_message, assert_warnings
-from sqlalchemy import *
-from sqlalchemy.orm import attributes
-from sqlalchemy import exc as sa_exc, event
-from sqlalchemy.orm import exc as orm_exc
-from sqlalchemy.orm import *
+from sqlalchemy import (
+    testing, exc as sa_exc, event, String, Column, Table, select, func)
+from sqlalchemy.testing import (
+    fixtures, engines, eq_, assert_raises, assert_raises_message,
+    assert_warnings, mock, expect_warnings)
+from sqlalchemy.orm import (
+    exc as orm_exc, Session, mapper, sessionmaker, create_session,
+    relationship, attributes)
 from sqlalchemy.testing.util import gc_collect
-from sqlalchemy import testing
-from sqlalchemy.testing import fixtures
-from sqlalchemy.testing import engines
 from test.orm._fixtures import FixtureTest
 
 
@@ -52,8 +50,8 @@ class SessionTransactionTest(FixtureTest):
         u = User(name='ed')
         sess.add(u)
         sess.flush()
-        sess.commit() # commit does nothing
-        trans.rollback() # rolls back
+        sess.commit()  # commit does nothing
+        trans.rollback()  # rolls back
         assert len(sess.query(User).all()) == 0
         sess.close()
 
@@ -67,7 +65,7 @@ class SessionTransactionTest(FixtureTest):
             conn = testing.db.connect()
             trans = conn.begin()
             sess = create_session(bind=conn, autocommit=False,
-                                autoflush=True)
+                                  autoflush=True)
             u1 = User(name='u1')
             sess.add(u1)
             sess.flush()
@@ -84,7 +82,43 @@ class SessionTransactionTest(FixtureTest):
             conn.close()
             raise
 
+    @testing.requires.savepoints
+    def test_nested_accounting_new_items_removed(self):
+        User, users = self.classes.User, self.tables.users
 
+        mapper(User, users)
+
+        session = create_session(bind=testing.db)
+        session.begin()
+        session.begin_nested()
+        u1 = User(name='u1')
+        session.add(u1)
+        session.commit()
+        assert u1 in session
+        session.rollback()
+        assert u1 not in session
+
+    @testing.requires.savepoints
+    def test_nested_accounting_deleted_items_restored(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        session = create_session(bind=testing.db)
+        session.begin()
+        u1 = User(name='u1')
+        session.add(u1)
+        session.commit()
+
+        session.begin()
+        u1 = session.query(User).first()
+
+        session.begin_nested()
+        session.delete(u1)
+        session.commit()
+        assert u1 not in session
+        session.rollback()
+        assert u1 in session
 
     @testing.requires.savepoints
     def test_heavy_nesting(self):
@@ -92,22 +126,50 @@ class SessionTransactionTest(FixtureTest):
 
         session = create_session(bind=testing.db)
         session.begin()
-        session.connection().execute(users.insert().values(name='user1'
-                ))
+        session.connection().execute(users.insert().values(
+            name='user1'))
         session.begin(subtransactions=True)
         session.begin_nested()
-        session.connection().execute(users.insert().values(name='user2'
-                ))
-        assert session.connection().execute('select count(1) from users'
-                ).scalar() == 2
+        session.connection().execute(users.insert().values(
+            name='user2'))
+        assert session.connection().execute(
+            'select count(1) from users').scalar() == 2
         session.rollback()
-        assert session.connection().execute('select count(1) from users'
-                ).scalar() == 1
-        session.connection().execute(users.insert().values(name='user3'
-                ))
+        assert session.connection().execute(
+            'select count(1) from users').scalar() == 1
+        session.connection().execute(users.insert().values(
+            name='user3'))
         session.commit()
-        assert session.connection().execute('select count(1) from users'
-                ).scalar() == 2
+        assert session.connection().execute(
+            'select count(1) from users').scalar() == 2
+
+    @testing.requires.savepoints
+    def test_dirty_state_transferred_deep_nesting(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        s = Session(testing.db)
+        u1 = User(name='u1')
+        s.add(u1)
+        s.commit()
+
+        nt1 = s.begin_nested()
+        nt2 = s.begin_nested()
+        u1.name = 'u2'
+        assert attributes.instance_state(u1) not in nt2._dirty
+        assert attributes.instance_state(u1) not in nt1._dirty
+        s.flush()
+        assert attributes.instance_state(u1) in nt2._dirty
+        assert attributes.instance_state(u1) not in nt1._dirty
+
+        s.commit()
+        assert attributes.instance_state(u1) in nt2._dirty
+        assert attributes.instance_state(u1) in nt1._dirty
+
+        s.rollback()
+        assert attributes.instance_state(u1).expired
+        eq_(u1.name, 'u1')
 
     @testing.requires.independent_connections
     def test_transactions_isolated(self):
@@ -126,9 +188,9 @@ class SessionTransactionTest(FixtureTest):
     @testing.requires.two_phase_transactions
     def test_twophase(self):
         users, Address, addresses, User = (self.tables.users,
-                                self.classes.Address,
-                                self.tables.addresses,
-                                self.classes.User)
+                                           self.classes.Address,
+                                           self.tables.addresses,
+                                           self.classes.User)
 
         # TODO: mock up a failure condition here
         # to ensure a rollback succeeds
@@ -137,7 +199,7 @@ class SessionTransactionTest(FixtureTest):
 
         engine2 = engines.testing_engine()
         sess = create_session(autocommit=True, autoflush=False,
-                            twophase=True)
+                              twophase=True)
         sess.bind_mapper(User, testing.db)
         sess.bind_mapper(Address, engine2)
         sess.begin()
@@ -150,6 +212,23 @@ class SessionTransactionTest(FixtureTest):
         assert users.count().scalar() == 1
         assert addresses.count().scalar() == 1
 
+    @testing.requires.independent_connections
+    def test_invalidate(self):
+        User, users = self.classes.User, self.tables.users
+        mapper(User, users)
+        sess = Session()
+        u = User(name='u1')
+        sess.add(u)
+        sess.flush()
+        c1 = sess.connection(User)
+
+        sess.invalidate()
+        assert c1.invalidated
+
+        eq_(sess.query(User).all(), [])
+        c2 = sess.connection(User)
+        assert not c2.invalidated
+
     def test_subtransaction_on_noautocommit(self):
         User, users = self.classes.User, self.tables.users
 
@@ -159,8 +238,8 @@ class SessionTransactionTest(FixtureTest):
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        sess.commit() # commit does nothing
-        sess.rollback() # rolls back
+        sess.commit()  # commit does nothing
+        sess.rollback()  # rolls back
         assert len(sess.query(User).all()) == 0
         sess.close()
 
@@ -239,7 +318,7 @@ class SessionTransactionTest(FixtureTest):
 
         u3 = User(name='u3')
         sess.add(u3)
-        sess.commit() # commit the nested transaction
+        sess.commit()  # commit the nested transaction
         sess.rollback()
 
         eq_(set(sess.query(User).all()), set([u2]))
@@ -309,13 +388,15 @@ class SessionTransactionTest(FixtureTest):
         sess = Session()
 
         to_flush = [User(name='ed'), User(name='jack'), User(name='wendy')]
+
         @event.listens_for(sess, "after_flush_postexec")
         def add_another_user(session, ctx):
             if to_flush:
                 session.add(to_flush.pop(0))
 
         x = [1]
-        @event.listens_for(sess, "after_commit")
+
+        @event.listens_for(sess, "after_commit")  # noqa
         def add_another_user(session):
             x[0] += 1
 
@@ -342,7 +423,6 @@ class SessionTransactionTest(FixtureTest):
             sess.commit
         )
 
-
     def test_error_on_using_inactive_session_commands(self):
         users, User = self.tables.users, self.classes.User
 
@@ -367,43 +447,41 @@ class SessionTransactionTest(FixtureTest):
         @event.listens_for(sess, "after_commit")
         def go(session):
             session.execute("select 1")
-        assert_raises_message(sa_exc.InvalidRequestError,
-                    "This session is in 'committed' state; no further "
-                    "SQL can be emitted within this transaction.",
-                    sess.commit)
+        assert_raises_message(
+            sa_exc.InvalidRequestError,
+            "This session is in 'committed' state; no further "
+            "SQL can be emitted within this transaction.",
+            sess.commit)
 
     def test_no_sql_during_prepare(self):
         sess = create_session(bind=testing.db, autocommit=False, twophase=True)
 
         sess.prepare()
 
-        assert_raises_message(sa_exc.InvalidRequestError,
-                    "This session is in 'prepared' state; no further "
-                    "SQL can be emitted within this transaction.",
-                    sess.execute, "select 1")
+        assert_raises_message(
+            sa_exc.InvalidRequestError,
+            "This session is in 'prepared' state; no further "
+            "SQL can be emitted within this transaction.",
+            sess.execute, "select 1")
 
     def test_no_prepare_wo_twophase(self):
         sess = create_session(bind=testing.db, autocommit=False)
 
         assert_raises_message(sa_exc.InvalidRequestError,
-                    "'twophase' mode not enabled, or not root "
-                    "transaction; can't prepare.",
-                    sess.prepare)
+                              "'twophase' mode not enabled, or not root "
+                              "transaction; can't prepare.",
+                              sess.prepare)
 
     def test_closed_status_check(self):
         sess = create_session()
         trans = sess.begin()
         trans.rollback()
         assert_raises_message(
-                sa_exc.ResourceClosedError,
-                "This transaction is closed",
-                trans.rollback
-        )
+            sa_exc.ResourceClosedError, "This transaction is closed",
+            trans.rollback)
         assert_raises_message(
-                sa_exc.ResourceClosedError,
-                "This transaction is closed",
-                trans.commit
-        )
+            sa_exc.ResourceClosedError, "This transaction is closed",
+            trans.commit)
 
     def test_deactive_status_check(self):
         sess = create_session()
@@ -412,9 +490,9 @@ class SessionTransactionTest(FixtureTest):
         trans2.rollback()
         assert_raises_message(
             sa_exc.InvalidRequestError,
-            "This Session's transaction has been rolled back by a nested "
-            "rollback\(\) call.  To begin a new transaction, issue "
-            "Session.rollback\(\) first.",
+            r"This Session's transaction has been rolled back by a nested "
+            r"rollback\(\) call.  To begin a new transaction, issue "
+            r"Session.rollback\(\) first.",
             trans.commit
         )
 
@@ -428,10 +506,10 @@ class SessionTransactionTest(FixtureTest):
             trans2.rollback(_capture_exception=True)
         assert_raises_message(
             sa_exc.InvalidRequestError,
-            "This Session's transaction has been rolled back due to a "
-            "previous exception during flush. To begin a new transaction "
-            "with this Session, first issue Session.rollback\(\). "
-            "Original exception was: test",
+            r"This Session's transaction has been rolled back due to a "
+            r"previous exception during flush. To begin a new transaction "
+            r"with this Session, first issue Session.rollback\(\). "
+            r"Original exception was: test",
             trans.commit
         )
 
@@ -450,48 +528,106 @@ class SessionTransactionTest(FixtureTest):
         )
         return sess, u1
 
+    def test_execution_options_begin_transaction(self):
+        bind = mock.Mock()
+        sess = Session(bind=bind)
+        c1 = sess.connection(execution_options={'isolation_level': 'FOO'})
+        eq_(
+            bind.mock_calls,
+            [
+                mock.call.contextual_connect(),
+                mock.call.contextual_connect().
+                execution_options(isolation_level='FOO'),
+                mock.call.contextual_connect().execution_options().begin()
+            ]
+        )
+        eq_(c1, bind.contextual_connect().execution_options())
+
+    def test_execution_options_ignored_mid_transaction(self):
+        bind = mock.Mock()
+        conn = mock.Mock(engine=bind)
+        bind.contextual_connect = mock.Mock(return_value=conn)
+        sess = Session(bind=bind)
+        sess.execute("select 1")
+        with expect_warnings(
+                "Connection is already established for the "
+                "given bind; execution_options ignored"):
+            sess.connection(execution_options={'isolation_level': 'FOO'})
+
     def test_warning_on_using_inactive_session_new(self):
         User = self.classes.User
 
         sess, u1 = self._inactive_flushed_session_fixture()
         u2 = User(name='u2')
         sess.add(u2)
+
         def go():
             sess.rollback()
         assert_warnings(go,
-            ["Session's state has been changed on a "
-            "non-active transaction - this state "
-            "will be discarded."],
-        )
+                        ["Session's state has been changed on a "
+                         "non-active transaction - this state "
+                         "will be discarded."],
+                        )
         assert u2 not in sess
         assert u1 in sess
 
     def test_warning_on_using_inactive_session_dirty(self):
         sess, u1 = self._inactive_flushed_session_fixture()
         u1.name = 'newname'
+
         def go():
             sess.rollback()
         assert_warnings(go,
-            ["Session's state has been changed on a "
-            "non-active transaction - this state "
-            "will be discarded."],
-        )
+                        ["Session's state has been changed on a "
+                         "non-active transaction - this state "
+                         "will be discarded."],
+                        )
         assert u1 in sess
         assert u1 not in sess.dirty
 
     def test_warning_on_using_inactive_session_delete(self):
         sess, u1 = self._inactive_flushed_session_fixture()
         sess.delete(u1)
+
         def go():
             sess.rollback()
         assert_warnings(go,
-            ["Session's state has been changed on a "
-            "non-active transaction - this state "
-            "will be discarded."],
-        )
+                        ["Session's state has been changed on a "
+                         "non-active transaction - this state "
+                         "will be discarded."],
+                        )
         assert u1 in sess
         assert u1 not in sess.deleted
 
+    def test_warning_on_using_inactive_session_rollback_evt(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        sess = Session()
+        u1 = User(id=1, name='u1')
+        sess.add(u1)
+        sess.commit()
+
+        u3 = User(name='u3')
+
+        @event.listens_for(sess, "after_rollback")
+        def evt(s):
+            sess.add(u3)
+
+        sess.add(User(id=1, name='u2'))
+
+        def go():
+            assert_raises(
+                orm_exc.FlushError, sess.flush
+            )
+
+        assert_warnings(go,
+                        ["Session's state has been changed on a "
+                         "non-active transaction - this state "
+                         "will be discarded."],
+                        )
+        assert u3 not in sess
+
     def test_preserve_flush_error(self):
         User = self.classes.User
 
@@ -499,14 +635,14 @@ class SessionTransactionTest(FixtureTest):
 
         for i in range(5):
             assert_raises_message(sa_exc.InvalidRequestError,
-                              "^This Session's transaction has been "
-                              r"rolled back due to a previous exception "
-                              "during flush. To "
-                              "begin a new transaction with this "
-                              "Session, first issue "
-                              r"Session.rollback\(\). Original exception "
-                              "was:",
-                              sess.commit)
+                                  "^This Session's transaction has been "
+                                  r"rolled back due to a previous exception "
+                                  "during flush. To "
+                                  "begin a new transaction with this "
+                                  "Session, first issue "
+                                  r"Session.rollback\(\). Original exception "
+                                  "was:",
+                                  sess.commit)
         sess.rollback()
         sess.add(User(id=5, name='some name'))
         sess.commit()
@@ -521,6 +657,35 @@ class SessionTransactionTest(FixtureTest):
         assert session.transaction is not None, \
             'autocommit=False should start a new transaction'
 
+    @testing.requires.python2
+    @testing.requires.savepoints_w_release
+    def test_report_primary_error_when_rollback_fails(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        session = Session(testing.db)
+
+        with expect_warnings(".*during handling of a previous exception.*"):
+            session.begin_nested()
+            savepoint = session.\
+                connection()._Connection__transaction._savepoint
+
+            # force the savepoint to disappear
+            session.connection().dialect.do_release_savepoint(
+                session.connection(), savepoint
+            )
+
+            # now do a broken flush
+            session.add_all([User(id=1), User(id=1)])
+
+            assert_raises_message(
+                sa_exc.DBAPIError,
+                "ROLLBACK TO SAVEPOINT ",
+                session.flush
+            )
+
+
 class _LocalFixture(FixtureTest):
     run_setup_mappers = 'once'
     run_inserts = None
@@ -530,10 +695,11 @@ class _LocalFixture(FixtureTest):
     def setup_mappers(cls):
         User, Address = cls.classes.User, cls.classes.Address
         users, addresses = cls.tables.users, cls.tables.addresses
-        mapper(User, users, properties={
-            'addresses':relationship(Address, backref='user',
-                                 cascade="all, delete-orphan",
-                                    order_by=addresses.c.id),
+        mapper(
+            User, users, properties={
+                'addresses': relationship(
+                    Address, backref='user', cascade="all, delete-orphan",
+                    order_by=addresses.c.id),
             })
         mapper(Address, addresses)
 
@@ -574,7 +740,9 @@ class FixtureDataTest(_LocalFixture):
 
         assert u1.name == 'will'
 
+
 class CleanSavepointTest(FixtureTest):
+
     """test the behavior for [ticket:2452] - rollback on begin_nested()
     only expires objects tracked as being modified in that transaction.
 
@@ -604,27 +772,31 @@ class CleanSavepointTest(FixtureTest):
 
     @testing.requires.savepoints
     def test_rollback_ignores_clean_on_savepoint(self):
-        User, users = self.classes.User, self.tables.users
+
         def update_fn(s, u2):
             u2.name = 'u2modified'
         self._run_test(update_fn)
 
     @testing.requires.savepoints
     def test_rollback_ignores_clean_on_savepoint_agg_upd_eval(self):
-        User, users = self.classes.User, self.tables.users
+        User = self.classes.User
+
         def update_fn(s, u2):
-            s.query(User).filter_by(name='u2').update(dict(name='u2modified'),
-                                    synchronize_session='evaluate')
+            s.query(User).filter_by(name='u2').update(
+                dict(name='u2modified'), synchronize_session='evaluate')
         self._run_test(update_fn)
 
     @testing.requires.savepoints
     def test_rollback_ignores_clean_on_savepoint_agg_upd_fetch(self):
-        User, users = self.classes.User, self.tables.users
+        User = self.classes.User
+
         def update_fn(s, u2):
-            s.query(User).filter_by(name='u2').update(dict(name='u2modified'),
-                                    synchronize_session='fetch')
+            s.query(User).filter_by(name='u2').update(
+                dict(name='u2modified'),
+                synchronize_session='fetch')
         self._run_test(update_fn)
 
+
 class ContextManagerTest(FixtureTest):
     run_inserts = None
     __backend__ = True
@@ -637,6 +809,7 @@ class ContextManagerTest(FixtureTest):
         mapper(User, users)
 
         sess = Session()
+
         def go():
             with sess.begin_nested():
                 sess.add(User())   # name can't be null
@@ -671,6 +844,7 @@ class ContextManagerTest(FixtureTest):
         mapper(User, users)
 
         sess = Session(autocommit=True)
+
         def go():
             with sess.begin():
                 sess.add(User())  # name can't be null
@@ -692,7 +866,7 @@ class AutoExpireTest(_LocalFixture):
     def test_expunge_pending_on_rollback(self):
         User = self.classes.User
         sess = self.session()
-        u2= User(name='newuser')
+        u2 = User(name='newuser')
         sess.add(u2)
         assert u2 in sess
         sess.rollback()
@@ -701,7 +875,7 @@ class AutoExpireTest(_LocalFixture):
     def test_trans_pending_cleared_on_commit(self):
         User = self.classes.User
         sess = self.session()
-        u2= User(name='newuser')
+        u2 = User(name='newuser')
         sess.add(u2)
         assert u2 in sess
         sess.commit()
@@ -799,8 +973,7 @@ class AutoExpireTest(_LocalFixture):
         u1.addresses.remove(a1)
 
         s.flush()
-        eq_(s.query(Address).filter(Address.email_address=='foo').all(),
-                [])
+        eq_(s.query(Address).filter(Address.email_address == 'foo').all(), [])
         s.rollback()
         assert a1 not in s.deleted
         assert u1.addresses == [a1]
@@ -814,7 +987,6 @@ class AutoExpireTest(_LocalFixture):
         sess.commit()
         eq_(u1.name, 'newuser')
 
-
     def test_concurrent_commit_pending(self):
         User = self.classes.User
         s1 = self.session()
@@ -823,12 +995,13 @@ class AutoExpireTest(_LocalFixture):
         s1.commit()
 
         s2 = self.session()
-        u2 = s2.query(User).filter(User.name=='edward').one()
+        u2 = s2.query(User).filter(User.name == 'edward').one()
         u2.name = 'will'
         s2.commit()
 
         assert u1.name == 'will'
 
+
 class TwoPhaseTest(_LocalFixture):
     __backend__ = True
 
@@ -844,6 +1017,7 @@ class TwoPhaseTest(_LocalFixture):
 
         assert u not in s
 
+
 class RollbackRecoverTest(_LocalFixture):
     __backend__ = True
 
@@ -876,7 +1050,7 @@ class RollbackRecoverTest(_LocalFixture):
         eq_(
             s.query(User).all(),
             [User(id=1, name='edward',
-                addresses=[Address(email_address='foober')])]
+                  addresses=[Address(email_address='foober')])]
         )
 
     @testing.requires.savepoints
@@ -906,9 +1080,10 @@ class RollbackRecoverTest(_LocalFixture):
         s.commit()
         eq_(
             s.query(User).all(),
-            [User(id=1, name='edward',
-                addresses=[Address(email_address='foober')])]
-        )
+            [
+                User(
+                    id=1, name='edward',
+                    addresses=[Address(email_address='foober')])])
 
 
 class SavepointTest(_LocalFixture):
@@ -928,18 +1103,19 @@ class SavepointTest(_LocalFixture):
         u1.name = 'edward'
         u2.name = 'jackward'
         s.add_all([u3, u4])
-        eq_(s.query(User.name).order_by(User.id).all(),
-                    [('edward',), ('jackward',), ('wendy',), ('foo',)])
+        eq_(
+            s.query(User.name).order_by(User.id).all(),
+            [('edward',), ('jackward',), ('wendy',), ('foo',)])
         s.rollback()
         assert u1.name == 'ed'
         assert u2.name == 'jack'
-        eq_(s.query(User.name).order_by(User.id).all(),
-                    [('ed',), ('jack',)])
+        eq_(
+            s.query(User.name).order_by(User.id).all(),
+            [('ed',), ('jack',)])
         s.commit()
         assert u1.name == 'ed'
         assert u2.name == 'jack'
-        eq_(s.query(User.name).order_by(User.id).all(),
-                    [('ed',), ('jack',)])
+        eq_(s.query(User.name).order_by(User.id).all(), [('ed',), ('jack',)])
 
     @testing.requires.savepoints
     def test_savepoint_delete(self):
@@ -969,19 +1145,23 @@ class SavepointTest(_LocalFixture):
         u1.name = 'edward'
         u2.name = 'jackward'
         s.add_all([u3, u4])
-        eq_(s.query(User.name).order_by(User.id).all(),
-                [('edward',), ('jackward',), ('wendy',), ('foo',)])
+        eq_(
+            s.query(User.name).order_by(User.id).all(),
+            [('edward',), ('jackward',), ('wendy',), ('foo',)])
         s.commit()
+
         def go():
             assert u1.name == 'edward'
             assert u2.name == 'jackward'
-            eq_(s.query(User.name).order_by(User.id).all(),
-                    [('edward',), ('jackward',), ('wendy',), ('foo',)])
+            eq_(
+                s.query(User.name).order_by(User.id).all(),
+                [('edward',), ('jackward',), ('wendy',), ('foo',)])
         self.assert_sql_count(testing.db, go, 1)
 
         s.commit()
-        eq_(s.query(User.name).order_by(User.id).all(),
-                [('edward',), ('jackward',), ('wendy',), ('foo',)])
+        eq_(
+            s.query(User.name).order_by(User.id).all(),
+            [('edward',), ('jackward',), ('wendy',), ('foo',)])
 
     @testing.requires.savepoints
     def test_savepoint_rollback_collections(self):
@@ -991,30 +1171,40 @@ class SavepointTest(_LocalFixture):
         s.add(u1)
         s.commit()
 
-        u1.name='edward'
+        u1.name = 'edward'
         u1.addresses.append(Address(email_address='bar'))
         s.begin_nested()
         u2 = User(name='jack', addresses=[Address(email_address='bat')])
         s.add(u2)
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'),
-                                        Address(email_address='bar')]),
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
                 User(name='jack', addresses=[Address(email_address='bat')])
-            ]
-        )
+            ])
         s.rollback()
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'),
-                                        Address(email_address='bar')]),
-            ]
-        )
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
+            ])
         s.commit()
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'),
-                                        Address(email_address='bar')]),
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
             ]
         )
 
@@ -1026,28 +1216,43 @@ class SavepointTest(_LocalFixture):
         s.add(u1)
         s.commit()
 
-        u1.name='edward'
+        u1.name = 'edward'
         u1.addresses.append(Address(email_address='bar'))
         s.begin_nested()
         u2 = User(name='jack', addresses=[Address(email_address='bat')])
         s.add(u2)
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), Address(email_address='bar')]),
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
                 User(name='jack', addresses=[Address(email_address='bat')])
             ]
         )
         s.commit()
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), Address(email_address='bar')]),
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
                 User(name='jack', addresses=[Address(email_address='bat')])
             ]
         )
         s.commit()
-        eq_(s.query(User).order_by(User.id).all(),
+        eq_(
+            s.query(User).order_by(User.id).all(),
             [
-                User(name='edward', addresses=[Address(email_address='foo'), Address(email_address='bar')]),
+                User(
+                    name='edward',
+                    addresses=[
+                        Address(email_address='foo'),
+                        Address(email_address='bar')]),
                 User(name='jack', addresses=[Address(email_address='bat')])
             ]
         )
@@ -1058,7 +1263,7 @@ class SavepointTest(_LocalFixture):
         sess = self.session()
 
         sess.begin_nested()
-        u2= User(name='newuser')
+        u2 = User(name='newuser')
         sess.add(u2)
         assert u2 in sess
         sess.rollback()
@@ -1091,7 +1296,8 @@ class AccountingFlagsTest(_LocalFixture):
         sess.add(u1)
         sess.commit()
 
-        testing.db.execute(users.update(users.c.name=='ed').values(name='edward'))
+        testing.db.execute(
+            users.update(users.c.name == 'ed').values(name='edward'))
 
         assert u1.name == 'ed'
         sess.expire_all()
@@ -1108,7 +1314,8 @@ class AccountingFlagsTest(_LocalFixture):
         u1.name = 'edwardo'
         sess.rollback()
 
-        testing.db.execute(users.update(users.c.name=='ed').values(name='edward'))
+        testing.db.execute(
+            users.update(users.c.name == 'ed').values(name='edward'))
 
         assert u1.name == 'edwardo'
         sess.expire_all()
@@ -1125,12 +1332,14 @@ class AccountingFlagsTest(_LocalFixture):
         u1.name = 'edwardo'
         sess.rollback()
 
-        testing.db.execute(users.update(users.c.name=='ed').values(name='edward'))
+        testing.db.execute(
+            users.update(users.c.name == 'ed').values(name='edward'))
 
         assert u1.name == 'edwardo'
         sess.commit()
 
-        assert testing.db.execute(select([users.c.name])).fetchall() == [('edwardo',)]
+        assert testing.db.execute(select([users.c.name])).fetchall() == \
+            [('edwardo',)]
         assert u1.name == 'edwardo'
 
         sess.delete(u1)
@@ -1139,8 +1348,9 @@ class AccountingFlagsTest(_LocalFixture):
     def test_preflush_no_accounting(self):
         User, users = self.classes.User, self.tables.users
 
-        sess = Session(_enable_transaction_accounting=False,
-                        autocommit=True, autoflush=False)
+        sess = Session(
+            _enable_transaction_accounting=False, autocommit=True,
+            autoflush=False)
         u1 = User(name='ed')
         sess.add(u1)
         sess.flush()
@@ -1153,7 +1363,8 @@ class AccountingFlagsTest(_LocalFixture):
         sess.rollback()
 
         sess.begin()
-        assert testing.db.execute(select([users.c.name])).fetchall() == [('ed',)]
+        assert testing.db.execute(select([users.c.name])).fetchall() == \
+            [('ed',)]
 
 
 class AutoCommitTest(_LocalFixture):
@@ -1183,6 +1394,7 @@ class AutoCommitTest(_LocalFixture):
         sess = create_session(autocommit=True)
 
         fail = False
+
         def fail_fn(*arg, **kw):
             if fail:
                 raise Exception("commit fails")
@@ -1214,6 +1426,7 @@ class AutoCommitTest(_LocalFixture):
         sess = create_session(autocommit=True)
 
         fail = False
+
         def fail_fn(*arg, **kw):
             if fail:
                 raise Exception("commit fails")
@@ -1259,14 +1472,13 @@ class AutoCommitTest(_LocalFixture):
         assert 'id' not in u1.__dict__
         eq_(u1.id, 3)
 
+
 class NaturalPKRollbackTest(fixtures.MappedTest):
     __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
-        Table('users', metadata,
-            Column('name', String(50), primary_key=True)
-        )
+        Table('users', metadata, Column('name', String(50), primary_key=True))
 
     @classmethod
     def setup_classes(cls):
@@ -1280,10 +1492,7 @@ class NaturalPKRollbackTest(fixtures.MappedTest):
 
         session = sessionmaker()()
 
-        u1, u2, u3= \
-            User(name='u1'),\
-            User(name='u2'),\
-            User(name='u3')
+        u1, u2, u3 = User(name='u1'), User(name='u2'), User(name='u3')
 
         session.add_all([u1, u2, u3])
 
@@ -1387,5 +1596,3 @@ class NaturalPKRollbackTest(fixtures.MappedTest):
         assert u2 not in s
 
         assert s.identity_map[(User, ('u1',))] is u1
-
-
