@@ -286,6 +286,10 @@ class DefaultRequirements(SuiteRequirements):
                     ("mysql", "<", (5, 0, 3)),
                     ], "savepoints not supported")
 
+    @property
+    def savepoints_w_release(self):
+        return self.savepoints + skip_if(
+            "oracle", "oracle doesn't support release of savepoint")
 
     @property
     def schemas(self):
@@ -389,10 +393,21 @@ class DefaultRequirements(SuiteRequirements):
             no_support('postgresql+zxjdbc',
                     'FIXME: JDBC driver confuses the transaction state, may '
                        'need separate XA implementation'),
-            exclude('mysql', '<', (5, 0, 3),
-                        'two-phase xact not supported by database'),
+            no_support('mysql',
+                'recent MySQL communiity editions have too many issues '
+                '(late 2016), disabling for now'),
             ])
 
+    @property
+    def two_phase_recovery(self):
+        return self.two_phase_transactions + (
+            skip_if(
+               "mysql",
+               "crashes on most mariadb and mysql versions"
+            )
+        )
+
+
     @property
     def views(self):
         """Target database must support VIEWs."""
@@ -668,6 +683,10 @@ class DefaultRequirements(SuiteRequirements):
                     ("mssql+pyodbc", None, None, "crashes due to bug #351"),
                 )
 
+    @property
+    def duplicate_key_raises_integrity_error(self):
+        return fails_on("postgresql+pg8000")
+
     @property
     def python2(self):
         return skip_if(
@@ -723,12 +742,12 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def range_types(self):
         def check_range_types(config):
-            if not against(config, "postgresql+psycopg2"):
+            if not against(
+                    config,
+                    ["postgresql+psycopg2", "postgresql+psycopg2cffi"]):
                 return False
             try:
-                config.db.execute("select '[1,2)'::int4range;")
-                # only supported in psycopg 2.5+
-                from psycopg2.extras import NumericRange
+                config.db.scalar("select '[1,2)'::int4range;")
                 return True
             except:
                 return False
@@ -760,6 +779,27 @@ class DefaultRequirements(SuiteRequirements):
             config.db.dialect._dbapi_version <= (1, 10, 1)
         )
 
+    @property
+    def psycopg2_native_json(self):
+        return self.psycopg2_compatibility
+
+    @property
+    def psycopg2_native_hstore(self):
+        return self.psycopg2_compatibility
+
+    @property
+    def psycopg2_compatibility(self):
+        return only_on(
+            ["postgresql+psycopg2", "postgresql+psycopg2cffi"]
+        )
+
+    @property
+    def psycopg2_or_pg8000_compatibility(self):
+        return only_on(
+            ["postgresql+psycopg2", "postgresql+psycopg2cffi",
+             "postgresql+pg8000"]
+        )
+
     @property
     def percent_schema_names(self):
         return skip_if(
@@ -806,16 +846,39 @@ class DefaultRequirements(SuiteRequirements):
             )
         )
 
+    @property
+    def no_mssql_freetds(self):
+        return self.mssql_freetds.not_()
+
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
         return skip_if(["oracle", "firebird"], "non-standard SELECT scalar syntax")
 
+    @property
+    def mysql_fsp(self):
+        return only_if('mysql >= 5.6.4')
 
     @property
     def mysql_fully_case_sensitive(self):
         return only_if(self._has_mysql_fully_case_sensitive)
 
+    @property
+    def mysql_zero_date(self):
+        def check(config):
+             row = config.db.execute("show variables like 'sql_mode'").first()
+             return not row or "NO_ZERO_DATE" not in row[1]
+
+        return only_if(check)
+
+    @property
+    def mysql_non_strict(self):
+        def check(config):
+             row = config.db.execute("show variables like 'sql_mode'").first()
+             return not row or "STRICT" not in row[1]
+
+        return only_if(check)
+
     def _has_mysql_on_windows(self, config):
         return against(config, 'mysql') and \
                 config.db.dialect._detect_casing(config.db) == 1
